<?
class ZvukogramList extends WP_List_Table
{
    public $plg_name;
	public $plg_dir;
	
	public $params         = [];
	public $found_data     = [];
	public $selected_data  = [];
	public $table_items    = [];
	public $cur_list_table = 'list_table';
	public $items_on_page  = 0;
	public $icon_yes       = '<span class="dashicons dashicons-yes-alt" style="color:#1fc41f;"></span>';
	public $icon_no        = '<span class="dashicons dashicons-dismiss" style="color:#e2e2e2;"></span>';

	public function __construct()
    {
        $name = explode( '/', plugin_basename(__DIR__) );
		$this->plg_name = strtolower( $name[0] );
		
		$this->plg_dir = plugin_dir_path( __DIR__ );
		
		parent::__construct( [ 'singular' => $this->cur_list_table, 'plural' => $this->cur_list_table, 'ajax' => true ] );
		
		$this->params = get_option( $this->plg_name . '_params' );
		$this->items_on_page  = $this->params['list_audio'];
    }

	public function prepare_items()
	{
		$columns  = $this->get_columns();
		$hidden   = $this->get_hidden_columns();
		$sortable = $this->get_sortable_columns();

		$this->_column_headers = [ $columns, $hidden, $sortable ];

		$this->process_bulk_action();

		$per_page     = $this->get_items_per_page( 'records_per_page', $this->items_on_page );
		$current_page = $this->get_pagenum();
		$query        = self::get_records( $per_page, $current_page );
		$total_items  = $query[1];

		$data = $query[0];

		$this->set_pagination_args( [ 'total_items' => $total_items, 'per_page' => $per_page ] );

		$res = [];
		if( $data )
		{
			$i = 0;
			foreach($data as $post)
			{
				$post_meta = get_post_meta( $post->ID );
				
				$title = '<span style="color:#999;" >' . __( '- without a name-', $this->plg_name ) . '</span>';
				if( $post->post_title ) $title = $post->post_title;
				
				$p = __( 'Post', $this->plg_name );
				if( $post->post_type != 'post' ) $p = __( 'Page', $this->plg_name );
				
				$css = '
					background:#e6e6e6;
					font-size:10px;
					color:#444;
					padding:2px 7px 2px 7px;
					margin-right:5px;
				';
				
				$audio_parts = '';
				
				$meta_plg_visible = $this->icon_no;
				if( @$post_meta['_' . $this->plg_name . '_visible'][0] ) $meta_plg_visible = $this->icon_yes;
				
				$date_add = @$post_meta['_' . $this->plg_name . '_date'][0];
				$file     = @$post_meta['_' . $this->plg_name . '_file'][0];
				
				$audio_set = '
					<span class="dashicons dashicons-controls-volumeoff" style="color:#b32d2e"></span> 
					<span style="color:#999;">'.__( 'Not announced', $this->plg_name ).'</span>';
				
				$bitrate = ( @$post_meta['_' . $this->plg_name . '_bitrate'][0] )? @$post_meta['_' . $this->plg_name . '_bitrate'][0] : $this->params['bitrate'];
				
				$sentence = ( @$post_meta['_' . $this->plg_name . '_sentence'][0] )? @$post_meta['_' . $this->plg_name . '_sentence'][0] : '';
				if( $sentence )
				{
					$sentence = '<a href="#" aria-label="'.__( 'Pauses between sentences', $this->plg_name ).': '.$sentence.'" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip" style="'.$css.'"><b>P1: </b>'.$sentence.'</a>';
				}
				
				$paragraph = ( @$post_meta['_' . $this->plg_name . '_paragraph'][0] )? @$post_meta['_' . $this->plg_name . '_paragraph'][0] : '';
				if( $paragraph )
				{
					$paragraph = '<a href="#" aria-label="'.__( 'Pauses between paragraphs', $this->plg_name ).': '.$paragraph.'" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip" style="'.$css.'"><b>P2: </b>'.$paragraph.'</a>';
				}
				
				$speed = ( @$post_meta['_' . $this->plg_name . '_speed'][0] )? @$post_meta['_' . $this->plg_name . '_speed'][0] : '';
				if( $speed )
				{
					$speed = '<a href="#" aria-label="'.__( 'Playback speed', $this->plg_name ).': '.$speed.'" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip" style="'.$css.'"><b>S: </b>'.$speed.'</a>';
				}
				
				$pitch = ( @$post_meta['_' . $this->plg_name . '_pitch'][0] )? @$post_meta['_' . $this->plg_name . '_pitch'][0] : '';
				if( $pitch )
				{
					$pitch = '<a href="#" aria-label="'.__( 'Voice pitch', $this->plg_name ).': '.$pitch.'" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip" style="'.$css.'"><b>V: </b>'.$pitch.'</a>';
				}
				
				if( $file )
				{
					$audio_set = 
					'
						<span class="dashicons dashicons-controls-volumeon" style="color:#03c03c;"></span> 
						<span style="font-size:12px;">
							<b>mp3</b>, &nbsp; Hz: '.$bitrate.'
							<div style="margin:5px 0 5px 0;">
								'.$sentence.$paragraph.$speed.$pitch.'
							</div>
						</span>
					';
				}

				$res[] = [
					'ID'               => $post->ID,
					'title'            => '<div style="color:#000;">'.$title.'</div>',
					'audio_set'        => $audio_set,
					'audio_lang'       => @$post_meta['_' . $this->plg_name . '_audio_lang'][0],
					'audio_voice'      => @$post_meta['_' . $this->plg_name . '_audio_voice'][0],
					'post_type'        => $p,
					'date_add'         => $date_add,
					$this->plg_name . '_status' => $meta_plg_visible
				];
				$i++;
			}
		}
		$this->items = $res;
	}

	function column_default( $item, $column_name )
	{
		foreach( $this->table_items as $tb => $name )
		{
			if( $column_name == $tb ) return $item[ $column_name ];
		}
		return print_r( $item, true ) ;
	}

	public static function get_records($per_page = 5, $page_number = 1)
	{
		global $wpdb;
		
		$orderby = ( ! empty($_GET['orderby']) ) ? $_GET['orderby'] : 'ID';
		$order   = ( ! empty($_GET['order']) ) ? $_GET['order'] : 'asc';
		
		$args['posts_per_page'] = $per_page;
		$args['post_type']      = [ 'post', 'page' ];
		$args['post_status']    = 'publish';
		$args['page']           = $page_number;
		$args['offset']         = ($page_number - 1) * $per_page;
		$args['orderby']        = $orderby;
		$args['order']          = $order;

		$query = new WP_Query( $args );

		return [ $query->posts, $query->found_posts ];
	}

	function get_columns()
	{
		$columns['cb'] = '<input type="checkbox" />';
		foreach( $this->table_items as $tb => $name )
		{
			$columns[$tb] = __( $name, $this->cur_list_table );
		}
		return $columns;
	}

	public function get_hidden_columns()
	{
		return [];
	}

	public function get_sortable_columns()
	{
		foreach( $this->table_items as $tb => $name )
		{
			$columns[$tb] = [ $tb, true ];
		}
		return $columns;
	}

	function column_cb( $item )
	{
		return sprintf( '<input type="checkbox" name="item_id[]" value="%s" />', $item['ID'] );
	}

	function column_title( $item )
	{
		$actions = [
			'pid' => 'ID: ' . $item['ID'],
			
			'audio' => '<a href="/wp-admin/admin.php?page='.$this->plg_name.'&tab=edit-'.$this->plg_name.'&id='.$item['ID'].'">'.__('Voice over', $this->plg_name).'</a>',
			
			'edit' => '<a href="/wp-admin/admin.php?page='.$this->plg_name.'&tab=edit-'.$this->plg_name.'&id='.$item['ID'].'">'.__('Edit', $this->plg_name).'</a>',

			'delete_audio' => sprintf('<a href="#" onclick="if(confirm(\''.__('Delete the voiceover and all related data?', $this->plg_name).'\')){ document.location.href=\'?page=%s&action=%s&item_id=%s\' }" style="color:#b32d2e;">'.__('Clear', $this->plg_name).'</a>', $_REQUEST['page'], 'delete_audio', $item['ID']),
			
			'go' => '<a href="'.get_permalink($item['ID']).'">'.__('Go over', $this->plg_name).'</a>',
		];
		return sprintf( '%1$s %2$s', $item['title'], $this->row_actions($actions) );
	}

	function get_bulk_actions()
	{
		$actions = [
			'delete_audio' => __( 'Remove the voiceover', $this->plg_name ),
			'hide_audio'   => __( 'Hide', $this->plg_name ),
			'show_audio'   => __( 'Show', $this->plg_name ),
		];
		return $actions;
	}
	
	function key_request($key)
	{
		if( array_key_exists($key, $_REQUEST) ) return $_REQUEST[ $key ]; 
		
		return '';
	}
	
	public function process_bulk_action()
	{
		$post_ids = $this->key_request( 'item_id' );
		if( ! is_array($post_ids) ) $post_ids = [ $post_ids ];

		if( 'delete_audio' === $this->current_action() && $post_ids )
		{
			foreach($post_ids as $post_id)
			{
				if( $post_id > 0 )
				{
					delete_post_meta( $post_id, '_edit_lock' );
					
					$file = get_post_meta( $post_id, '_' . $this->plg_name . '_file', true );
					if( $file )
					{
						$f = explode('/', $file);
						
						if( end($f) )
						{
							if( file_exists(  $this->plg_dir . 'uploads/' . end($f) ) ) unlink(  $this->plg_dir . 'uploads/' . end($f) );
						}
					}
					
					$meta_remove = get_post_meta( $post_id );
					foreach( $meta_remove as $key => $item ) 
					{
						$m = explode('_' . $this->plg_name . '_', $key);
													
						if( ! empty( @$m[1] ) ) delete_post_meta( $post_id, $key );
					}
					
					if( file_exists(  $this->plg_dir . 'logs/' . $post_id . '.txt' ) ) unlink(  $this->plg_dir . 'logs/' . $post_id . '.txt' );
				}
			}
			
			//echo '<script>document.location.reload();</script>';
		}
		else if( 'hide_audio' === $this->current_action() && $post_ids )
		{
			foreach($post_ids as $post_id)
			{
				if( $post_id > 0 ) update_post_meta( $post_id, '_' . $this->plg_name . '_visible', 0 );
			}
			echo '<script>document.location.reload();</script>';
		}
		else if( 'show_audio' === $this->current_action() && $post_ids )
		{
			foreach($post_ids as $post_id)
			{
				if( $post_id > 0 ) update_post_meta( $post_id, '_' . $this->plg_name . '_visible', 1 );
			}
			echo '<script>document.location.reload();</script>';
		}
	}

	function no_items()
	{
		_e( 'No records found',  $this->plg_name );
	}
}



