<?php

if( ! defined( 'ABSPATH' ) ) exit;

class WP_ZVUKOGRAM 
{
	public $plg_name        = 'zvukogram';
	public $plg_name_menu   = 'Zvukogram';
	public $class_list_name = 'ZvukogramList';
	
	public $ssml_link     = 'https://zvukogram.com/node/ssml/';
	public $url_voices    = 'https://zvukogram.com/index.php?r=api/voices';
	public $url_balance   = 'https://zvukogram.com/index.php?r=api/balance';
	public $url_api_home  = 'https://zvukogram.com';
	public $url_long_text = 'https://zvukogram.com/index.php?r=api/longtext';
	public $url_result    = 'https://zvukogram.com/index.php?r=api/result';
	public $url_dictors   = 'https://zvukogram.com/rating/';
	public $demo_audio    = 'example_news.mp3';
	
	public $plg_dir;
	public $plg_url;
	
	public $current_version;
    public $plugin_slug;
	public $current_tab = 'homepage';
	
	// Default list params
	public $audio_format_arr    = [ 'mp3', 'wav', 'ogg' ];
	public $audio_emotion_arr   = [ 'good', 'evil', 'neutral' ];
	public $audio_bitrate_arr   = [ 192000, 176400, 96000, 88200, 64000, 48000, 44100, 32000, 22050, 16000, 11025, 8000 ];
	public $audio_sentence_arr  = [ 150, 200, 250, 300, 350, 400, 500, 600, 700, 800, 900, 1000, 1250, 1500, 2000, 3000, 4000, 5000 ];
	public $audio_paragraph_arr = [ 150, 200, 250, 300, 350, 400, 500, 600, 700, 800, 900, 1000, 1250, 1500, 2000, 3000, 4000, 5000 ];
	public $audio_speed_arr     = [ 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2.0 ];
	public $audio_pitch_arr     = [ -20, -19, -18, -17, -16, -15, -14, -13, -12, -11, -10, -9, -8, -7, -6, -5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,  17, 18, 19, 20 ];
	
	// Default sound config
	public $audio_defailt_set = [
		'audio_lang'  => 'Русский', 
		'audio_voice' => 'Александр',
		'bitrate'     => 48000, 
		'sentence'    => 300, 
		'paragraph'   => 400, 
		'speed'       => 1, 
		'pitch'       => 0,
		'set_text'    => '1',
		'doc_type'    => 'post', 
		'list_docs'   => '50', 
		'list_audio'  => '20', 
		'format'      => 'mp3',
		'emotion'     => 'good',
		'balans'      => ''
	];
	
	private $iam_root           = false;
	private $this_admin_page    = false;
	private $params             = [];
	private $php_min_version    = '6.1';
	private $wp_min_version     = '5.5';
	private $id                 = 0;
	private $children_id        = 0;
	private $children_post_type = '';
	
	// Log file set
	private $request_log        = true;

	private $allowed_html = [
		'a' => [
			'href'  => true,
			'title' => true,
		],
		'br'     => [],
		'em'     => [],
		'strong' => [],
		'i'      => [],
		'b'      => [],
		'ul'     => [],
		'li'     => [],
		'div'    => [],
		'span'   => [],
		'p'      => [],
		'img'    => [
			'src' => true
		]
	];

	function __construct()
	{
		$this->plg_dir  = plugin_dir_path( __DIR__ );
		$this->plg_url  = plugin_dir_url( __DIR__ );
						
		$this->init_params();
		
		add_action( 'init', [ $this, 'language_init' ] );
		
		if( ! $this->check_version_php() || ! $this->check_version_wp() ) return;
		
		add_action( 'plugins_loaded', [$this, 'admin_page_view'] );
		
		add_action( 'wp_ajax_select_voice', [ $this, 'select_voice_func_ajax' ] );
		
		add_action( 'wp_ajax_get_doc', [ $this, 'get_doc_func_ajax' ] );
		
		add_action( 'wp_ajax_sound_start', [ $this, 'sound_start_func_ajax' ] );
		
		add_action( 'wp_ajax_sound_process', [ $this, 'sound_process_func_ajax' ] );
		
		add_action( 'wp_ajax_upload_audio', [ $this, 'upload_audio_func_ajax' ] );
		
		add_action( 'wp_ajax_remove_audio', [ $this, 'remove_audio_func_ajax' ] );
		
		add_action( 'wp_ajax_preview_player', [ $this, 'preview_player_func_ajax' ] );
		
		add_action( 'add_meta_boxes' , [ $this, 'audio_meta_boxes' ] );
		
		add_shortcode( $this->plg_name, [ $this, 'cur_plg_short_func' ] );
	}
	
	// * Params
	function init_params()
	{
		$this->plugin_slug = $this->plg_name.'/'.$this->plg_name.'.php';

		$cfg = get_option( $this->plg_name . '_params' );
		
		// * Default set after activate
		if( ! $cfg ) 
		{
			$cfg = $this->audio_defailt_set;
						
			// * Lang array
			$cfg['voices_arr'] = json_decode( file_get_contents( $this->url_voices ) );
						
			update_option( $this->plg_name . '_params', $cfg );
		}
		$this->params = $cfg;
	}
	
	// * Key in array
	function key_array($arr, $key)
	{
		if( is_array($arr) && array_key_exists($key, $arr) ) return $arr[$key]; 
		else return '';
	}
	
	// * Request clear
	private function data_cut_request( $key )
	{
		$str = $this->key_array( $_REQUEST, $key );
		$str = str_replace( "'", "", $str );
		$str = sanitize_text_field( $str );
		return $str;
	}
	
	// * Lang init
	public function language_init()
	{
		$this->iam_root = current_user_can('administrator');
		
		// * in admin panel
		$this->this_admin_page = is_admin();
		
		load_plugin_textdomain( $this->plg_name, false, $this->plg_name.'/lang' );
	}
	
	// * Check РНР
	private function check_version_php(){
		
		if( version_compare( PHP_VERSION, $this->php_min_version, '<' ) )
		{
			$this->view_notice( 
				'notice-error', 
				__( 'Minimum required PHP version ( or later ) ', $this->plg_name ) . $this->php_min_version
			);
			return false;
		}
		return true;
	}
	
	// * Check WP
	private function check_version_wp(){
		
		if( version_compare( PHP_VERSION, $this->wp_min_version, '<' ) )
		{
			$this->view_notice( 
				'notice-error', 
				__( 'Minimum required WP version ( or later ) ', $this->plg_name ) . $this->wp_min_version
			);
			return false;
		}
		return true;
	}
	
		
	// * Show saved notice
	function view_saved_notice()
	{
		$transient_notice_arr = get_transient( 'ran_notice' );

		if( isset($transient_notice_arr[0]) && isset($transient_notice_arr[1]) ) 
		{
			$this->view_notice( $transient_notice_arr[0], $transient_notice_arr[1] );
			
			delete_transient( 'ran_notice' );
		}
	}
	
	// * Save notice
	public function transient_notice( $type,  $msg )
	{
		set_transient( 'ran_notice', [ $type, $msg ], 60 );
	}
	
	
	// * View notice
	public function view_notice( $type,  $msg )
	{
		echo '<div class="notice '.$type.'"><p>'.$msg.'</p></div>';
	}
	
	// * Admin check
	function check_admin()
	{
		if( ! $this->iam_root && ! $this->this_admin_page ) return;
	}
	
	// * Get posts from meta fields
	private function get_posts_parent_id( $post_id, $post_type, $limit )
	{
		$args = [
			'posts_per_page' => $limit,
			'post_status'    => 'publish',
			'post_type'      => $this->plg_name . '-' . $post_type,
			'meta_query'	 => [
				[
					'key'     => '_parent_id',
					'value'   => $post_id,
					'compare' => '='
				]
			],
		];
		$qwery = new WP_Query( $args );
		
		return $qwery->posts;
	}
	
	// * Metabox add
	function audio_meta_boxes()
	{
		add_meta_box(
			'audio_metabox', 
			__( 'Voice the text', $this->plg_name ),
			[$this, 'view_audio_meta_boxes'],
			['page', 'post'],
			'side',
			'high'
		);
	}
	
	// * Metabox view
	function view_audio_meta_boxes()
	{
		global $post;
		
		if($post->post_status === 'publish')
		{
			echo '
				<a class="button action button-primary " style="border-radius:3px;padding:2px 10px;" href="/wp-admin/admin.php?page='.$this->plg_name.'&tab=edit-'.$this->plg_name.'&id='.$post->ID.'" >
					<span class="dashicons-before dashicons-format-audio" style="margin:5px 5px 0 0;font-size:10px;padding:0;display:inline-block;"></span>
					'.__( 'Go to voiceover', $this->plg_name ).'
				</a>';
		}
		else
		{
			echo '<b>'.__( 'The document has not been published.', $this->plg_name ).'</b><br><br>'.__( 'Publish the text so that it becomes available for voice-over.', $this->plg_name );
		}
	}
	
	// * Admin config
	public function admin_page_view()
	{
		$this->check_admin();	
		
		add_action( 'admin_menu', [$this, 'admin_page_view_menu'] );
		add_filter( 'plugin_action_links_' . $this->plugin_slug, [$this, 'admin_plugin_link'], 10, 2 );
	}

	// * Menu
	public function admin_page_view_menu()
	{
		$this->check_admin();
		
		add_menu_page(  
			$this->plg_name_menu,
			$this->plg_name_menu,
			'administrator',          
			$this->plg_name,                   
			[$this, 'admin_plugin_page'],
			'dashicons-format-audio', 9
		);  
	}
	
	// * Link panel plugin
	public function admin_plugin_link( $links, $file )
	{
		$this->check_admin();
		
		array_unshift( 
			$links, 
			'<a href="admin.php?page=' . $this->plg_name . '&tab=settings">' . __( 'Plugin settings', $this->plg_name ) . '</a>' 
		);
		return $links;
	}
	
	// * Save configuration
	function save_config()
	{
		$this->check_admin();

		$this->params['api_key']    = $this->data_cut_request( 'api_key' );
		$this->params['user_email'] = $this->data_cut_request( 'user_email' );
		
		$this->params['audio_lang']  = $this->data_cut_request( 'audio_lang' );
		$this->params['audio_voice'] = $this->data_cut_request( 'audio_voice' );
		$this->params['format']      = $this->data_cut_request( 'format' );
		
		$this->params['bitrate']   = $this->data_cut_request( 'bitrate' );
		$this->params['sentence']  = $this->data_cut_request( 'sentence' );
		$this->params['paragraph'] = $this->data_cut_request( 'paragraph' );
		$this->params['speed']     = $this->data_cut_request( 'speed' );
		$this->params['pitch']     = $this->data_cut_request( 'pitch' );
		$this->params['emotion']   = $this->data_cut_request( 'emotion' );
		
		$this->params['set_title']   = $this->data_cut_request( 'set_title' );
		$this->params['set_excerpt'] = $this->data_cut_request( 'set_excerpt' );
		$this->params['set_text']    = $this->data_cut_request( 'set_text' );
		
		
		$this->params['code_player'] = htmlspecialchars( $_REQUEST['code_player'] );
		$this->params['code_css']    = htmlspecialchars( $_REQUEST['code_css'] );
		$this->params['list_docs']   = (int) $this->data_cut_request( 'list_docs' );
		$this->params['list_audio']  = (int) $this->data_cut_request( 'list_audio' );
		$this->params['player']      = (int) $this->data_cut_request( 'player' );
		
		$error = '';
		if( $this->params['api_key'] && $this->params['user_email'] )
		{
			$data = [ 
				'email' => $this->params['user_email'],
				'token' => $this->params['api_key'] 
			];
			
			// Check balance
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_URL, $this->url_balance );
			curl_setopt($ch, CURLOPT_HEADER, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
			
			$response = curl_exec($ch);
			$response = json_decode($response, true);
			
			if( $response['status'] == 1 )
			{
				$this->params['balans']   = $response['balans'];
			}
			else
			{
				$error = __( 'API error', $this->plg_name ) . ': ' . $response['error'];
				$this->params['balans'] = '';
			}
		}
		
		// * Update options
		update_option( $this->plg_name . '_params', $this->params );
		
		// * View
		if( ! $error ) 
		{
			$this->view_notice( 'notice-success', __( 'Changes saved successfully', $this->plg_name ) );
		}
		else
		{
			$this->view_notice( 'notice-error', $error );
		}
	}

	function html_tab_start( $tab_name, $tab_slug, $form_name )
	{
		return '
			<div class="wrap wrap-h">	
				<div class="h-b">'.$tab_name.'</div>
				<div style="float:left;border-left:1px solid #e2e2e2;padding:0 0 0 30px;">
					<form action="'.esc_url( admin_url( 'admin.php?page=' . $this->plg_name . '&tab='.$tab_slug ) ).'" name="'.$form_name.'" class="'.$form_name.'" method="post" >
						
		';
	}

	function html_table_start()
	{
		return '<table style="border:0 !important;">';
	}

	function html_table_end()
	{
		return '</table>';
	}
	
	function html_tab_end( $html, $requires )
	{
		$req = '<br>' . __( 'Fields marked with * are required to be filled in', $this->plg_name );
		if( ! $requires ) $req = '';
		
		return '
						' . $req . '
						<br>
						<br>
						'.$html.'
					</form>
				</div>
				<div style="clear:both;"></div>
			</div>
			<script>
				var lang_plg = "'.$this->plg_name.'";
				var lang_wait_full = "'.__( 'Wait for the full completion of the voiceover process', $this->plg_name ).'";
				var lang_insert_post_id = "'.__( 'Specify the post_id you need separated by commas', $this->plg_name ).'";
				var lang_select_docs = "'.__( 'Select the required documents', $this->plg_name ).'";
				var lang_upload_file = "'.__( 'Uploading a file...', $this->plg_name ).'";
				var lang_finished = "'.__( 'Dubbing has been completed successfully, files have been saved and uploaded to the site', $this->plg_name ).'";
				var lang_finished_error = "'.__( 'Dubbing is completed with errors, check all settings and run again', $this->plg_name ).'";
				var lang_reload_page = "'.__( 'Update the page to view the changes', $this->plg_name ).'";
			</script>
			<script src="'.$this->plg_url.'js/plg.js"></script>
		';
	}

	function html_tr_wrapper( $html )
	{
		return '
			<tr>
				<td colspan="2">
					<div class="sub_txt">'.$html.'</div>
				</td>
			</tr>
		';
	}

	function html_tr_wrapper_empty( $html )
	{
		return '
			<tr>
				<td colspan="2">' . $html . '</td>
			</tr>
		';
	}

	function html_hidden( $name, $value )
	{
		return '<input type="hidden" class="'.$name.'" name="'.$name.'" value="'.$value.'">';
	}

	function html_btn_submit( $name, $attr )
	{
		return '<button type="submit" class="button button-primary new-btn" '.$attr.'>' . $name . ' </button>';
	}

	function html_tab_box( $arr )
	{
		$html = '';
		$i = 0;
		foreach( $arr as $name )
		{
			$active = '';
			if( ! $i ) $active = 'tab-box-select-active';
			
			$html .= '<div class="tab-box-select '.$active.'" data-name="1">'.$name.'</div>';
			$i++;
		}
		return '
			<div class="tab-box">
				<div class="tab-box-block-select">'.$html.'</div>
				<div class="tab-panels">
		';
	}

	function html_tab_box_end()
	{
		return '
				</div>
				<div class="cls"></div>
			</div>
			<script>
				jQuery( ".t-box" ).each( function( i,v ) {
					var act = "";
					if(i == 0) act = "t-box-active";
					jQuery(this).addClass("t-box-"+i+" "+act);
				});
				
				jQuery( ".tab-box-select" ).each( function( i,v ) {
					var t = jQuery(this).text();
					jQuery(".t-box-"+i+" .h-b").text(t);
				});
			</script>
		';
	}

	function html_tab_box_element()
	{
		return '
			<div class="t-box" >
				<div class="h-b" ></div>
		';
	}

	function html_tab_box_element_end()
	{
		return '</div>';
	}
	
	// * elements table generate
	function html_tr( $args )
	{
		$out = '<tr '.$this->key_array( $args, 'tr_html' ).'><td>';
		
		$microtip = '';
		
		if( $this->key_array( $args, 'microtip' ) ) 
		{
			$microtip ='<sup aria-label="'.$this->key_array( $args, 'microtip' ).'" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip">?</sup>';
		}
		
		if( $this->key_array( $args, 'title' ) ) 
		{
			$required = '';
			if( $this->key_array( $args, 'attr' ) == 'required' ) $required = ' <spab style="color:#999;">*</span>';
			
			$out .=	'
				<div class="a_label '.$this->key_array( $args, 'title_add_class' ).'">
					'.$this->key_array( $args, 'title' ) . $required . ' &nbsp;'.$microtip.'
				</div>
				' . $this->key_array( $args, 'title_add_html' );
		}

		$out .= '</td><td>';
		
		if( $this->key_array( $args, 'field' ) == 'input' )
		{
			$out .= '
				<div class="m-labels">
					<input 
						type       ="'.$this->key_array( $args, 'type' ).'" 
						class      ="a_inp '.$this->key_array( $args, 'name' ).' '.$this->key_array( $args, 'field_add_class' ).'" 
						name       ="'.$this->key_array( $args, 'name' ).'" 
						placeholder="'.$this->key_array( $args, 'placeholder' ).'" 
						value      ="'.$this->key_array( $args, 'value' ).'" 
						'.$this->key_array( $args, 'attr' ).' 
					>'
					. $this->key_array( $args, 'field_add_html' ).'
				</div>
			';
		}
		else if( $this->key_array( $args, 'field' ) == 'textarea' )
		{
			$out .= '
					<div class="m-labels">
						<textarea 
							class="a_inp a_inp_textarea '.$this->key_array( $args, 'name' ).' '.$this->key_array( $args, 'field_add_class' ).'" 
							name="'.$this->key_array( $args, 'name' ).'" 
							placeholder="'.$this->key_array( $args, 'placeholder' ).'" 
							'.$this->key_array( $args, 'attr' ).' 
							>'
							. $this->key_array( $args, 'value' )
							.'</textarea>'
							. $this->key_array( $args, 'field_add_html' ).'
					</div>
				';
		}
		else if( $this->key_array( $args, 'field' ) == 'select' )
		{
			$out .= '
				<div class="m-labels">
					<select 
						class="a_inp '.$this->key_array( $args, 'name' ).' '.$this->key_array( $args, 'field_add_class' ).'" 
						name="'.$this->key_array( $args, 'name' ).'" 
						'.$this->key_array( $args, 'attr' ).' 
					>';
					
			if( $this->key_array( $args, 'select_arr' ) )
			{
				foreach( $this->key_array( $args, 'select_arr' ) as $data )
				{
					$grey_css = '';
					if( $data[2] == 'disabled' ) $grey_css = 'style="color:#999;"';
					
					$s = '';
					if( $this->key_array( $args, 'selected' ) == $data[0] ) $s = 'selected';
					
					$out .= '<option value="'.$data[0].'" '.$s.' '.$data[2].' '.$grey_css.'>'.$data[1].'</option>';
				}
			}
			$out .= '
					</select>
					'. $this->key_array( $args, 'field_add_html' ).'
				</div>' 
				 ;
		}
		else if( $this->key_array( $args, 'field' ) == 'radio' )
		{
			$out .= '<div class="m-labels m-labels-radio">';
			if( $this->key_array( $args, 'select_arr' ) )
			{
				foreach( $this->key_array( $args, 'select_arr' ) as $data )
				{
					$s = '';
					if( $this->key_array( $args, 'selected' ) == $data[0] ) $s = 'checked';
					
					$uniq = 'z' . md5( uniqid() );
					
					$grey_css = '';
					if( $data[2] == 'disabled' ) $grey_css = 'label_grey';
					
					$out .= '
						<input type="radio" id="'.$uniq.'" name="'.$this->key_array( $args, 'name' ).'" value="'.$data[0].'" '.$s.' '.$data[2].' >
						<label for="'.$uniq.'" class="'.$grey_css.'">'.$data[1].'</label>'
						.$data[3];
				}
			}
			$out .= '</div>' . $this->key_array( $args, 'field_add_html' ) ;
		}
		
		$out .= '</td></tr>';
		return $out;
	}

	// * Generate table list
	private function generate_list( $ClassName, $class_file, $fields_array, $tab_name, $btn_name, $btn_action  )
	{
		require_once ( $this->plg_dir . 'includes/' . $class_file );
		$ClassName = new $ClassName();
		$ClassName->table_items = $fields_array;
		?>
			<div class="wrap wrap-h">
				<div class="h-b" style="float:left;margin-right:20px;"><?php echo $tab_name;?></div>
				
				<a href="/wp-admin/admin.php?page=<?php echo $this->plg_name;?>&tab=<?php echo $btn_action;?>" class="button action button-primary " style="float:left;margin-top:-9px;">
					<?php echo $btn_name;?>
				</a>
				
				<div class="cls"></div>
				
				<?php $ClassName->prepare_items(); ?>
				
				<form method="post" style="background:#fff;">
					<input type="hidden" name="page" value="<?php echo $this->plg_name;?>">
					<?php $ClassName->display(); ?>
				</form>
			</div>
		<?php 
	}
	
	// * Admin html header
	function html_plugin_header()
	{
		$this->check_admin();

		return '
			<script src="'.$this->plg_url.'js/jquery.js"></script>
			<script src="'.$this->plg_url.'js/jquery.modal.min.js"></script>
			
			<link rel="stylesheet" href="'.$this->plg_url.'css/style.css">
			<link rel="stylesheet" href="'.$this->plg_url.'css/jquery.modal.min.css">
			<link rel="stylesheet" href="'.$this->plg_url.'css/microtip.css">
			
			<link rel="preconnect" href="https://fonts.googleapis.com">
			<link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
			<link href="https://fonts.googleapis.com/css2?family=Montserrat:wght@100;200;300;400;500;600;700;800&display=swap" rel="stylesheet">
						
			<div class="wrap" style="background:#fff;padding:15px;">
				<h1>'.get_admin_page_title().'</h1>
		';
	}
	
	// * Form data load
	private function load_data_tab()
	{
		$this->check_admin();
		
		$data = [];

		$qpost = get_post( $this->id );
		$qpost_meta = get_post_meta( $this->id );
		
		if( $qpost && $qpost_meta )
		{
			$data['title'] = $qpost->post_title;
			foreach( $qpost_meta as $key => $value )
			{
				$data[ substr($key, 1) ] = $value[0];
			}
		}
		$data[ 'post_content' ] = $qpost->post_content;
		
		// * Cheldrens
		$data[ 'children_posts' ] = [];
		$data[ 'children_metas' ] = [];
		
		if( $this->children_post_type ) 
		{
			$qposts = $this->get_posts_parent_id( $this->id, $this->children_post_type, 1 );
			if( $qposts ) 
			{
				$data[ 'children_posts' ] = $qposts;
				$data[ 'children_metas' ] = get_post_meta( $qposts[0]->ID );
			}
		}
		return $data;
	}
	
	// * Save forms
	private function save_tab()
	{
		$this->check_admin();

		$type_tab   = $this->data_cut_request( 'save_tab' );
		$return_url = $this->data_cut_request( 'return_url' );
		
		if( $type_tab == $this->plg_name )
		{
			$post_id = (int) $this->data_cut_request( 'id' );
			$text    = htmlspecialchars( @$_REQUEST['edit_area_1'][0] );
			
			$this->save_audio_params_post( $post_id, $text );
			
			update_post_meta( $post_id, '_' . $this->plg_name . '_visible', $this->data_cut_request( $this->plg_name . '_visible' ) );
			
			$msg = __( 'Changes saved successfully', $this->plg_name );
			$this->transient_notice( 'notice-success', $msg );
			
			echo '<script>document.location.href="'.$return_url.'"</script>';
			exit;
		}
	}
	
	private function insert_post( $id, $meta )
	{
		$this->check_admin();
		
		$post_id = 0;
		
		// * Update
		if( $id > 0 )
		{
			// * Clear meta
			$meta_remove = get_post_meta( $id );
            foreach( $meta_remove as $key => $item ) 
			{
                delete_post_meta( $id, $key );
            }
			// * Update post
			$post_id = wp_update_post(
                [
					'ID'         => $id,
					'post_title' => $meta['title']
                ]
            );
		}
		else
		{
            // * New post
			$post_id = wp_insert_post( 
				[
					'post_title'  => $meta['title'],
					'post_type'   => $meta['post_type'],
					'post_status' => 'publish',
					'post_author' => get_current_user_id()
				]
			);
		}
		
		// * Update meta
		if( $post_id > 0 )
		{
			foreach( $meta as $key => $val ) 
			{
				update_post_meta( $post_id, '_' . $key, $val );
			}
		}
		
		$msg = __( 'Changes saved successfully', $this->plg_name );
		if( ! $meta['return_url'] )
		{
			$this->view_notice( 'notice-success', $msg );
		}
		else
		{
			$this->transient_notice( 'notice-success', $msg );
			
			echo '<script>document.location.href="/wp-admin/admin.php?page='.$this->plg_name.'&tab='.$meta['return_url'].'"</script>';
			exit;
		}
	}
	
	// * Admin view
	public function admin_plugin_page()
	{
		$this->check_admin();
		
		global $pagenow;
		
		$tab_values = [];
		
		$this->id = (int) $this->data_cut_request('id');
		
		// * View message
		$this->view_saved_notice();

		if( $pagenow == 'admin.php' && $this->data_cut_request('page') == $this->plg_name )
		{
			// * Save plugin config
			if( $this->data_cut_request('save_params') == 1 ) $this->save_config();
			
			// * Save tab
			if( $this->data_cut_request('save_tab') ) $this->save_tab();
			
			// * Current tab
			if( $this->data_cut_request('tab') ) $this->current_tab = $this->data_cut_request('tab');

			// * Header
			echo $this->html_plugin_header();
			
			// * Title tabs
			echo $this->view_tab();
			
			// * Select tabs
			switch ( $this->current_tab )
			{
				// * Create table
				case 'homepage' :
					$this->generate_list( 
						$this->class_list_name, 
						'class-'.$this->plg_name.'-list.php', 
						[
							'title'            => __( 'Headline', $this->plg_name ), 
							'audio_set'        => __( 'Voiced by ', $this->plg_name ),
							'audio_lang'       => __( 'Language', $this->plg_name ),
							'audio_voice'      => __( 'Voice', $this->plg_name ),
							'post_type'        => __( 'Type', $this->plg_name ), 
							'date_add'         => __( 'Date of the voiceover', $this->plg_name ),
							$this->plg_name . '_status' => __( 'Visibility', $this->plg_name )
						], 
						__( 'Documents', $this->plg_name ), 
						__( 'Mass dubbing', $this->plg_name ), 
						'add-'.$this->plg_name  
					);
				break;
				
				// * Add
				case 'add-'.$this->plg_name :
					
					$type_tab = $this->plg_name;
					
					if( $this->id > 0 ) $tab_values = $this->load_data_tab();

					echo $this->html_tab_start( __( 'To announce the documents', $this->plg_name ), 'add-' . $type_tab, 'form_' . $type_tab );

					// * Vertical tabs title
					echo $this->html_tab_box( 
						[ 
							__( 'Document Selection', $this->plg_name ),
							__( 'Audio Configuration', $this->plg_name ),
							__( 'Voiceover Sections', $this->plg_name )
						]
					);
					
					// * Tab 
					echo $this->html_tab_box_element();
					echo $this->html_table_start();
					
					echo $this->html_tr(
						[
							'title'           => __( 'Document selection method', $this->plg_name ),
							'field'           => 'select',
							'select_arr'      => [
								[ '0', __( 'Select from the list', $this->plg_name ), '' ],
								[ '1', __( 'Specify the POST_ID separated by commas', $this->plg_name ), '' ]
							],
							'selected'        => '',
							'microtip' => __( 'When selecting from the list, you can select the necessary documents.', $this->plg_name ),
							'name'            => 'select_method',
						]
					);
					
					echo $this->html_tr(
						[
							'title'           => __( 'Document type WP', $this->plg_name ),
							'field'           => 'select',
							'select_arr'      => [
								[ 'post', __( 'Post (Post)', $this->plg_name ), '' ],
								[ 'page', __( 'Page (Page)', $this->plg_name ), '' ]
							],
							'selected'        => '',
							'name'            => 'doc_type',
							'microtip' => __( 'Source of documents for voice-over.', $this->plg_name ),
							'tr_html' => 'class="vdoc_select"'
						]
					);
					
					// * pages
					$q['posts_per_page'] = $this->params['list_docs'];
					$q['post_type']      = 'page';
					$q['post_status']    = 'publish';
					
					$q['meta_query'] = [
						'relation' => 'OR',
						[
							'key'   => '_' . $this->plg_name . '_file',
							'value' => false,
							'type'  => 'BOOLEAN'
						],
						[
							'key'     => '_' . $this->plg_name . '_file',
							'compare' => 'NOT EXISTS'
						]
					];
					$query = new WP_Query( $q );
					$option_select_page = '';
					if( $query->posts )
					{
						foreach( $query->posts as $post )
						{
							$option_select_page .= '<option value="'.$post->ID.'">'.$post->post_title.'</option>';
						}
					}
					
					// * Posts
					$q['posts_per_page'] = $this->params['list_docs'];
					$q['post_type']      = 'post';
					$q['post_status']    = 'publish';
					
					$q['meta_query'] = [
						'relation' => 'OR',
						[
							'key'   => '_' . $this->plg_name . '_file',
							'value' => false,
							'type'  => 'BOOLEAN'
						],
						[
							'key'     => '_' . $this->plg_name . '_file',
							'compare' => 'NOT EXISTS'
						]
					];
					$query = new WP_Query( $q );
					$option_select_post = '';
					if( $query->posts )
					{
						foreach( $query->posts as $post )
						{
							$option_select_post .= '<option value="'.$post->ID.'">'.$post->post_title.'</option>';
						}
					}
					
					$def_data = $option_select_post;
					if( @$this->params['doc_type'] == 'page' ) $def_data = $option_select_page;
					
					?>
					<tr class="vpost_id" style="display:none;">
						<td>
							<div class="a_label"><?php echo __( 'POST_ID separated by commas', $this->plg_name ); ?></div>
						</td>
						<td>
							<div class="m-labels">
								<textarea class="a_inp ids_val" style="height:100px;" name="ids_val"></textarea>
							</div>
						</td>
					</tr>
					<tr class="vpost_select">
						<td>
							<div class="a_label"> 
								<?php echo __( 'Select the required documents', $this->plg_name ); ?> 
								&nbsp;<sup aria-label="<?php echo __( 'Shown:', $this->plg_name ) . ' ' . $this->params['list_docs'] . ', ' . __( 'you can change the number of displayed documents in the list in the plugin settings.', $this->plg_name ); ?>" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip">?</sup>
							</div>
						</td>
						<td>
							<div class="m-labels">
								<select class="a_inp multi_posts_id" multiple="multiple" style="height:200px;" name="multi_posts_id[]" >
									<?php echo $def_data; ?>
								</select>
							</div>
						</td>
					</tr>
					<div class="option_select_post" style="display:none;"><?php echo $option_select_post; ?></div>
					<div class="option_select_page" style="display:none;"><?php echo $option_select_page; ?></div>
					
					<?php
										
					echo $this->html_table_end();
										
					?>
					<div class="button button-primary save-btn" onclick="get_doc();" style="margin:20px 0 20px 0;" ><?php echo __( 'Preprocessing', $this->plg_name ); ?></div>
					
					<div class="post_target"></div>
					<div class="sound_target" id="sound_target"></div>
					
					<?php
					
					echo $this->html_tab_box_element_end();
					
					// * Vertical tab 
					echo $this->html_tab_box_element();
					echo $this->html_table_start();
					
					echo $this->get_data_audio([]);
					
					echo $this->html_table_end();
					echo $this->html_tab_box_element_end();
					
					// * Vertical tab
					echo $this->html_tab_box_element();
					echo $this->html_table_start();
					
					echo $this->get_data_section_doc([]);
					
					echo $this->html_table_end();
					echo $this->html_tab_box_element_end();
					
					// * Close vertical tab
					echo $this->html_tab_box_end();

					echo $this->html_tab_end( '', 0);
					
				break;
								
				// * Plugin config
				case 'settings' :
					echo $this->html_tab_start( __( 'Plugin settings', $this->plg_name ), 'settings', 'form_params' );
					echo $this->html_table_start();
					
					echo $this->html_tr_wrapper( __( 'API Server Connection Settings ', $this->plg_name ) . '<a href="'.$this->url_api_home.'" target="_blank">'.$this->url_api_home.'</a>' );
					
					echo $this->html_tr(
						[
							'title' => __( 'Your API token', $this->plg_name ),
							'field' => 'input',
							'type'  => 'text',
							'name'  => 'api_key',
							'value' => $this->key_array($this->params, 'api_key')
						]
					);
					
					echo $this->html_tr(
						[
							'title' => __( 'Email from personal account', $this->plg_name ),
							'field' => 'input',
							'type'  => 'text',
							'name'  => 'user_email',
							'value' => $this->key_array($this->params, 'user_email')
						]
					);
					
					$bl = __( 'Updated when saving settings and voicing', $this->plg_name );
					
					if( $this->key_array($this->params, 'balans') > 0 ) $bl = '<span style="color:green;font-weight:bold;">' .$this->key_array($this->params, 'balans') .'</span>';
					
					echo '
						<tr>
							<td>
								<div class="a_label ">'.__( 'Current Balance', $this->plg_name ).'</div>
							</td>
							<td>
								<div class="m-labels">'.$bl.'</div>
							</td>
						</tr>
					';
										
					echo $this->html_tr_wrapper( __( 'Default audio configuration', $this->plg_name ) );
					
					echo $this->get_data_audio([]);
					
					echo $this->html_tr_wrapper( __( 'Default voiceover sections', $this->plg_name ) );
					
					echo $this->get_data_section_doc([]);
				
					echo $this->html_tr_wrapper( __( 'Player Code Settings', $this->plg_name ) );

					$format_arr = [];
					foreach( $this->audio_format_arr as $v )
					{
						$format_arr[] = [ $v, $v, '' ];
					}
					echo $this->html_tr(
						[
							'title'           => __( 'File format', $this->plg_name ),
							'field'           => 'select',
							'select_arr'      => $format_arr,
							'selected'        => $this->key_array( $this->params, 'format'),
							'name'            => 'format'
						]
					);
					
					$code_css = '';
					
					echo $this->html_tr(
						[
							'title' => __( 'CSS player', $this->plg_name ),
							'field' => 'textarea',
							'name'  => 'code_css',
							'value' => stripcslashes( htmlspecialchars_decode( @$this->params['code_css'] ) ),
							'microtip' => __( 'Before the player code, your:', $this->plg_name ) . ' <style>...</style>',
							'attr' => 'style="height:150px;"'
						]
					);
					
					echo $this->html_tr(
						[
							'title'           => __( 'Player', $this->plg_name ),
							'field'           => 'select',
							'select_arr'      => [
								[ '0', __( 'HTML 5 audio', $this->plg_name ), '' ],
								[ '1', __( 'WP player', $this->plg_name ), '' ]
							],
							'selected'        => $this->key_array( $this->params, 'player'),
							'name'            => 'player'
						]
					);
					
$code = '<div class="zg-player">
<audio preload="auto" controls controlsList="nodownload">
	<source src="[AUDIO_FILE_PATH]">
</audio>
</div>
';

$code_wp = '[audio src=[AUDIO_FILE_PATH]]';
		
					$code_player = $code;
					if( $this->key_array($this->params, 'code_player') ) 
					{
						$code_player = stripcslashes( htmlspecialchars_decode( $this->params['code_player'] ) );
					}
					
					echo $this->html_tr(
						[
							'title' => __( 'Change the code', $this->plg_name ),
							'field' => 'textarea',
							'name'  => 'code_player',
							'value' => $code_player,
							'microtip' => __( 'If you have your own player loaded, add the full paths to the js and css files to the code and replace the executable code according to the instructions for your player, and the source of the audio file in it (for example: file_name.mp3) replace with [AUDIO_FILE_PATH]', $this->plg_name ),
							'attr' => 'style="height:150px;"',
							'field_add_html' => '<div class="default_code_div" ><a href="javascript:void(0);" class="default_code" style="text-decoration:none;display:inline-block;border-bottom:1px dashed #999;" >'.__( 'Restore the default code', $this->plg_name ).'</a></div>'
						]
					);
					
					echo '
						<tr>
							<td>
								<div class="a_label ">'.__( 'Result of code execution', $this->plg_name ).' &nbsp;
									<sup aria-label="'.__( 'If the HTML code is formed correctly, the sample audio file will be played. For WP player, the appearance is determined by the installed theme and may differ from the current one.', $this->plg_name ).'" class="microtp" data-microtip-position="top-right" data-microtip-size="large" role="tooltip">?</sup>
								</div>
							</td>
							<td valign="top"><div class="code_target" style="width:400px;"></div></td>
						</tr>
					';
					
					echo $this->html_tr_wrapper( __( 'Display Settings', $this->plg_name ) );
					
					echo $this->html_tr(
						[
							'title'           => __( 'of documents in the list', $this->plg_name ),
							'field'           => 'input',
							'type'            => 'number',
							'name'            => 'list_docs',
							'microtip' => __( 'How many pieces? show for selection in the list', $this->plg_name ),
							'value'           => $this->key_array( $this->params, 'list_docs'),
						]
					);
					
					echo $this->html_tr(
						[
							'title'           => __( 'Voiced documents', $this->plg_name ),
							'field'           => 'input',
							'type'            => 'number',
							'name'            => 'list_audio',
							'microtip' => __( 'How many pieces? show in the section - Documents', $this->plg_name ),
							'value'           => $this->key_array( $this->params, 'list_audio'),
						]
					);
					
					echo $this->html_hidden( 'example_audio', $this->plg_url.'uploads/' . $this->demo_audio );
					echo $this->html_hidden( 'default_player', base64_encode($code) );
					echo $this->html_hidden( 'default_player_wp', base64_encode($code_wp) );
					
					echo $this->html_hidden( 'save_params', '1' );
					echo $this->html_table_end();
					
					$code = '<?php echo do_shortcode( "['.$this->plg_name.']" ); ?>';
					ob_start();
						highlight_string($code);
						$cv = ob_get_contents();
					ob_end_clean();
					
					echo $this->html_tab_end( 
						$this->html_btn_submit( __( 'Save changes', $this->plg_name ), '' )
						.'
							
							
							
							
							<div style="font-size:15px;margin-top:0px;line-height:1.7em">
							<br>
							'.__( 'To display the audio player with sound recording on the page in the browser, add a shortcode to the document in the right place:', $this->plg_name ).' 
							<br>
							<b>['.$this->plg_name.']</b>
							<br>
							'.__( 'If the theme supports editing by Gutenberg, Elementor, etc. similarly, specify the theme editor in the desired template:', $this->plg_name ).'
							 <b> ['.$this->plg_name.']</b>
							<br>
							<br>
							'.__( 'If the theme is without an editor, enter the php code in the desired theme (template) file:', $this->plg_name ).'
							<br>
								'.$cv.'
						</div>
						',
						0
					);
				break;
				
				// * Edit from docs
				case 'edit-'.$this->plg_name :
					
					$tab_values = [];
					
					if( $this->id > 0 ) $tab_values = $this->load_data_tab();
					
					$t = __( 'Create/change the sound of the document', $this->plg_name );
					if( @$tab_values['title'] ) $t = '<div style="font-weight:500;margin:0px 0 20px 0;">' . $tab_values['title'] .'</div>';
						
					echo $this->html_tab_start( $t, 'edit', 'form_'.$this->plg_name );
					
					if( $this->key_array( $tab_values, $this->plg_name . '_file') )
					{
						$bl = $this->params['balans'];
						
						$bl_css = '#ffd1d9';
						if( $this->params['balans'] > 0 ) 
						{
							$bl_css = '#98fb98';
						}
						else
						{
							$bl = __( 'Unavailable', $this->plg_name );
						}
						
						echo '
							<h3>'.__( 'The document has been announced', $this->plg_name ).'</h3>
							<div style="">
								<div>
									<audio preload="auto" controls controlsList="nodownload">
										<source src="'.$tab_values[$this->plg_name . '_file'].'">
									</audio>
								</div>
								<div style="line-height:1.7em;margin-top:10px;">
									<b>'.__( 'Current Balance', $this->plg_name ).'</b> &nbsp;<span style="background:'.$bl_css.';padding:3px 7px;border-radius:3px;font-size:12px;">'.$bl.'</span>
									<br>
									<b>'.__( 'Link to the audio file', $this->plg_name ).':</b> <a href="'.$tab_values[$this->plg_name . '_file'].'" target="_blank">'.$tab_values[$this->plg_name . '_file'].'</a>
									<br>
									<b>'.__( 'Date of the voiceover', $this->plg_name ).':</b> '.$tab_values[$this->plg_name . '_date'].'
								</div>
							</div>
						';
					}
					
					$path = $this->plg_url . '/logs/' . $this->id . '.txt';
					$lg = '';
					if( file_exists( $this->plg_dir . '/logs/' . $this->id . '.txt' ) ) 
					{
						echo '
								<b>'.__( 'API Request Log', $this->plg_name ).':</b> <a href="'.$path.'" target="_blank">'.__( 'view the log', $this->plg_name ).'</a><br><br>
						';
					}
					
					echo $this->html_tab_box( 
						[ 
							__( 'Document', $this->plg_name ),
							__( 'Audio Configuration', $this->plg_name ),
							__( 'Voiceover Sections', $this->plg_name )
						]
					);
			
					// * Vertical tab 
					echo $this->html_tab_box_element();
						
						$ch = '';

						$current_content_md5 = ( $this->key_array( $tab_values, 'post_content' ) ) ? md5( $tab_values['post_content'] ) : '';
						$saved_content_md5   = $this->key_array( $tab_values, $this->plg_name . '_content_md5' );

						if( $current_content_md5 && $saved_content_md5 && $current_content_md5 != $saved_content_md5 ) 
						{
							$ch .= __( 'The original document (or its html) was changed in the WP editor.', $this->plg_name );
							$ch .= '<br>';
						}
						
						$current_title_md5 = ( $this->key_array( $tab_values, 'title' ) ) ? md5( $tab_values['title'] ) : '';
						$saved_title_md5   = $this->key_array( $tab_values, $this->plg_name . '_title_md5' );
												
						$current_excerpt     = $this->excerpt_post( $this->id );
						$current_excerpt_md5 = ( $current_excerpt ) ? md5( $current_excerpt ) : '';
						$saved_excerpt_md5   = $this->key_array( $tab_values, $this->plg_name . '_excerpt_md5' );
												
						$change = '';
						$edit = '';
						
						if( $this->key_array( $tab_values, $this->plg_name . '_text_saved' ) && $this->key_array( $tab_values, $this->plg_name . '_text_saved' ) != $this->key_array( $tab_values, $this->plg_name . '_text' )  )
						{
							$edit = '
								<div style="font-size:14px;padding:15px;border:1px dashed #9933ff;">
									<span style="color:#9933ff;font-weight:bold;">' 
										. __( 'The saved version was changed, but not announced.', $this->plg_name ). ' 
									</span>
									<div style="margin-top:5px;">
										<b>' . __( 'Previously announced text:', $this->plg_name ) .'</b>
										<textarea class="a_inp a_inp_textarea" style="width:100%; height:50px;">'.$this->key_array( $tab_values, $this->plg_name . '_text_saved' ).'</textarea>
									</div>
								</div>
							';
						}
																	
						if( $ch )
						{
							$change = '
								<div style="font-size:14px;padding:15px;border:1px dashed red;margin-bottom:10px;">
									<span style="color:red;font-weight:bold;">' . $ch . ' ' . '</span>' .__( 'Compare the current text for voicing and the Text (original) for changes.', $this->plg_name ) . ' <br>'.__( 'If necessary, make changes and re-voice it.', $this->plg_name ).'
								</div>
							';
						}
												
						echo '
							'.$change.$edit.'
							<div style="margin-bottom:10px;">
								<h3>'.__( 'Memo', $this->plg_name ).'</h3>
								<b>'.__( 'Current text for voice-over', $this->plg_name ).'</b> - '.__( 'the text that will be sent for dubbing is as it is.', $this->plg_name ).'<br>
								<b>'.__( 'Current text for voice-over (saved version)', $this->plg_name ).'</b> - '.__( 'similar, but a saved or previously voiced version is shown.', $this->plg_name ).'<br>
								<b>'.__( 'HTML (original)', $this->plg_name ).'</b> - '.__( 'code of the current document.', $this->plg_name ).'<br>
								<b>'.__( 'Text (original)', $this->plg_name ).'</b> - '.__( 'always contains the cleared text of the current HTML code (original). Appears after the changes are voiced or saved.', $this->plg_name ).'<br>
							</div>
							
							<div class="post_target"></div>
							<div class="sound_target" id="sound_target"></div>
							<input type="hidden" class="select_method" name="select_method" value="1" />
							<input type="hidden" class="is_edit_page" name="is_edit_page" value="" />
							<input type="hidden" class="ids_val" name="ids_val" value="'.$this->id.'" />
						';

					echo $this->html_tab_box_element_end();
					
					// * Vertical tab 
					echo $this->html_tab_box_element();
					echo $this->html_table_start();
						
					echo $this->get_data_audio($tab_values);
						
					echo $this->html_table_end();
					echo $this->html_tab_box_element_end();
					
					// * Vertical tab 
					echo $this->html_tab_box_element();
					echo $this->html_table_start();
						
					echo $this->get_data_section_doc($tab_values);
					echo $this->html_table_end();
					
					echo '
						<div style="margin-top:20px;">
							1. '.__( 'If the document was previously saved or voiced, this group of settings will be applied to the "Text (original)" tab', $this->plg_name ).' <br> 
							2. '.__( 'When trying to select all three options in "no", the "Voice text" section is automatically activated.', $this->plg_name ).'
						</div>
						';
						
					echo $this->html_tab_box_element_end();
					
					// * Vertical tab 
					echo $this->html_tab_box_element();
					echo $this->html_table_start();
					
					echo $this->html_tr(
						[
							'title'           => __( 'Visibility', $this->plg_name ),
							'field'           => 'select',
							'select_arr'      => [
								[ '1', __( 'Show audio player', $this->plg_name ), '' ],
								[ '0', __( 'Hide', $this->plg_name ), '' ]
							],
							'selected'        => @$tab_values[$this->plg_name . '_visible'],
							'name'            => $this->plg_name . '_visible'
						]
					);
						
					echo $this->html_table_end();
					echo $this->html_tab_box_element_end();
										
					// * Close vertical tab 
					echo $this->html_tab_box_end();
					
					echo $this->html_hidden( 'save_tab', $this->plg_name );
					echo $this->html_hidden( 'id', $this->id );
					echo $this->html_hidden( 'return_url', '/wp-admin/admin.php?page='.$this->plg_name.'&tab=edit-'.$this->plg_name.'&id=' . $this->id );
					
					$remove = '';
					if( $this->key_array( $tab_values, $this->plg_name . '_file') )
					{
						$remove = '	
								<div class="button " style="float:right;" onclick="if( confirm(\''.__( 'Delete voiceover and related data?', $this->plg_name ).'\') ){ remove_audio('.$this->id.'); }">' . __( 'Delete voiceover and audio file', $this->plg_name ) . ' </div>
						';
					}
					
					echo $this->html_tab_end( 
					'
							<script>
								$( document ).ready(function() {
									get_doc();
									$(".a_inp.set_title").attr("onchange", "pre_get_doc();" );
									$(".a_inp.set_excerpt").attr("onchange", "pre_get_doc();" );
									$(".a_inp.set_text").attr("onchange", "pre_get_doc();" );
								});
							</script>
							<button type="submit" class="button button-primary new-btn" style="float:left;">' . __( 'Save changes*', $this->plg_name ) . ' </button>
							'.$remove.'
							<div class="cls"></div>
							<div style="margin-top:20px;">* '.__( 'You can save changes to settings and text without the voice-over process. Also, all parameters and text are saved when you start the voiceover.', $this->plg_name ).'</div>
						',
						0
					);
				break;
			}
		}
	}
	
	// * Title post
	function excerpt_post( $post_id )
	{
		$ex = '';
		if( has_excerpt( $post_id ) ) $ex = get_the_excerpt( $post_id );
		
		return $ex;
	}
	
	// * Tabs names
	function view_tab() 
	{
		$this->check_admin();

		$tabs = [];
		$tabs['homepage'] = __( 'Documents', $this->plg_name );
		$tabs['settings'] = __( 'Plugin settings', $this->plg_name );
		
		// * data add|change view tab
		if( $this->current_tab == 'add-'.$this->plg_name ) $tabs['add-'.$this->plg_name] = __( 'Mass dubbing', $this->plg_name );
		if( $this->current_tab == 'edit-'.$this->plg_name ) $tabs['edit-'.$this->plg_name] = __( 'Create/change voiceover', $this->plg_name );

		$out = '';
		foreach( $tabs as $tab => $name )
		{
			$class = ( $tab == $this->current_tab ) ? ' nav-tab-active' : '';
			if( $this->id > 0 && ($tab !== 'homepage' || $tab !== 'plg-config') )
			{
				$tab .= '&id=' . $this->id;
			}
			$out .= "<a class='nav-tab$class' href='?page=".$this->plg_name."&tab=$tab'>$name</a>";
		}
		return '<div class="nav-tab-wrapper" style="margin-bottom:10px; background:#fff; ">'.$out.'</div>';
	}
	
	// * Clear html
	function clear_html( $content )
	{
		// check link
		preg_match_all('/<a[^>]*?>(.*?)<\/a>/si', $content, $matches);
		if( @$matches[1] )
		{
			foreach($matches[1] as $val)
			{
				if( preg_match('|^http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i', trim($val) ) ) 
				{
					$content = str_replace( $val, '', $content );
				}
			}
		}
		
		$content = str_replace( '(', '@-KL-@', $content );
		$content = str_replace( ')', '@-KR-@', $content );
		
		$content = preg_replace('#\[caption(.*?)\](.*?)\[/caption\]#is', ' ', $content);
		
		$content = preg_replace('/\[(.|\s)*?\]/', ' ', $content);
		
		$content = preg_replace("/[\r\n]+/m", "\n", $content);
		$content = preg_replace("/ {2,}/", " ", $content);
		
		$content = preg_replace('/<!--(.|\s)*?-->/', ' ', $content);
		$content = preg_replace('/\s*\([^)]*\)/', ' ', $content);
		
		$content = preg_replace('#<svg(.*?)>(.*?)</svg>#is', ' ', $content);
		$content = preg_replace('#<dl(.*?)>(.*?)</dl>#is', ' ', $content);
		$content = preg_replace('#<dd(.*?)>(.*?)</dd>#is', ' ', $content);
		$content = preg_replace('#<dt(.*?)>(.*?)</dt>#is', ' ', $content);
		$content = preg_replace('#<form(.*?)>(.*?)</form>#is', ' ', $content); 
		$content = preg_replace('#<noscript(.*?)>(.*?)</noscript>#is', ' ', $content);
		$content = preg_replace('#<button(.*?)>(.*?)</button>#is', ' ', $content);
		$content = preg_replace('#<textarea(.*?)>(.*?)</textarea>#is', ' ', $content);
		$content = preg_replace('#<code(.*?)>(.*?)</code>#is', ' ', $content);

		$content = str_replace('<p>', ' ', $content);
		$content = str_replace('</p>', "\r\n", $content);

		$content = str_replace('<br>', "\r\n", $content);
		$content = str_replace('<br />', "\r\n", $content);
		$content = str_replace('<br/>', "\r\n", $content);
		
		$content = str_replace('<li>', ' ', $content);
		$content = str_replace('</li>', "\r\n", $content);
		
		// *Text delete in link 
		$content = preg_replace('/\b((https?|ftp|file):\/\/|www\.)[-A-Z0-9+&@#\/%?=~_|$!:,.;]*[A-Z0-9+&@#\/%=~_|$]/i', ' ', $content);
		
		$content = preg_replace('#<[^>]+>#', ' ', $content);
		
		$content = wp_kses( wp_unslash($content), [] );
		
		$content = strip_tags($content);		
		
		$content = preg_replace( "/&#?[a-z0-9]+;/i", " ", $content ); // smiles html
				
		$content = preg_replace('/[^\p{L}0-9\+\-\_:\.@\\n \#\*\&\%\=\@\!\:\;\,\?\$\(\)]/u', "", $content ); // special chars
		
		$content = str_replace( '@-KL-@', "(", $content );
		$content = str_replace( '@-KR-@', ")", $content );
				
		$content = trim( preg_replace('/\t+/', ' ', $content) );
		
		$content = preg_replace('/\n{2,}/', "\n", $content);
		$content = preg_replace('/(\n)\1+/', '$1', $content);
		
		$content = str_replace("\n ", "\n", $content);
		
		$content = preg_replace("/ {2,}/", " ", $content);
		$content = preg_replace("/\n{2,}/", "\n", $content);
		
		$content = str_replace("\n ", "\n", $content);
		$content = preg_replace("/\n{2,}/", "\n", $content);
		
		$content = str_replace("\n", "\n\n", $content);
				
		return $content;
	}
	
	// * Ajax docs search
	function get_doc_func_ajax()
	{
		$this->check_admin();
		
		$audio_voice = $this->data_cut_request( 'audio_voice' );
		$bitrate     = $this->data_cut_request( 'bitrate' );
		$sentence    = $this->data_cut_request( 'sentence' );
		$paragraph   = $this->data_cut_request( 'paragraph' );
		$speed       = $this->data_cut_request( 'speed' );
		$pitch       = $this->data_cut_request( 'pitch' );
		
		$set_title   = $this->data_cut_request( 'set_title' );
		$set_excerpt = $this->data_cut_request( 'set_excerpt' );
		$set_text    = $this->data_cut_request( 'set_text' );
		
		$select_method = (int) $this->data_cut_request( 'select_method' );
		
		$ids_val   = $this->data_cut_request( 'ids_val' );
		$posts_arr = @$_POST['multi_posts_id'];
		
		if( $select_method ) $posts_arr = explode (',', $ids_val );
				
		$current = $this->data_cut_request( 'current' );
		
		$html = '';
		
		if( ! is_array($posts_arr) ) $posts_arr = [ $posts_arr ];
				
		$result = '
			<h3>'.__( 'Text preprocessing', $this->plg_name ).'</h3>
			<div>
			<span style="font-weight:bold;font-size:14px;">
			'
			.__( 'Before changing the text, set the desired audio configuration and the voiceover section.', $this->plg_name )
			.'</span><br>'
			.__( 'You can edit the text for voicing at your discretion.', $this->plg_name )
			.'<br>'
			
			.__( 'Special SSML markup is allowed in the text ', $this->plg_name )
			.' <a href="'.$this->ssml_link.'" target="_blank">'.$this->ssml_link.'</a></div>
			
			<div class="button button-primary save-btn snd-start" onclick="sound_start();" style="margin:10px 0 0 0;float:right;background:green;">'.__( 'Start voiceover', $this->plg_name ).'</div>
			
			<div class="cls"></div>
		';
		
		if( is_array($posts_arr) )
		{
			$posts = get_posts( 
				[
					'numberposts'      => $this->params['list_docs'], // * counts
					'include'          => $posts_arr,
					'post_type'        => ['post', 'page'],
					'suppress_filters' => true,
				]
			);
			global $post;

			if($posts)
			{
				$i = 1;
				foreach( $posts as $post )
				{
					$content = '';
					$orig    = '';
					$secions = '';
					
					if( $set_title && $post->post_title ) 
					{
						$secions .= $post->post_title . " \r\n";
						$orig .= '<!-- '.__( 'Added title', $this->plg_name ).' -->' . "\r\n"  . $post->post_title . "\r\n\r\n";
					}
					
					$excerpt = $this->excerpt_post( $post->ID );
					if( $set_excerpt && $excerpt ) 
					{
						$secions .= $excerpt;
						$orig .=  '<!-- '.__( 'Added excerpt', $this->plg_name ).' -->' . "\r\n"  . $excerpt . "\r\n\r\n";
					}
					
					if( $set_text && $post->post_content ) 
					{
						$content = $post->post_content;
						$orig .=  '<!-- '.__( 'Content Content', $this->plg_name ).' -->' . "\r\n" . $post->post_content . "\r\n";
					}
					
					if( $secions || $content )
					{
						// * Clear tag <textarea>
						$orig = preg_replace('#<textarea(.*?)>(.*?)</textarea>#is', ' ', $orig);
						
						// * changes
						$content = $this->clear_html( $content );
						$secions = $this->clear_html( $secions );
						
						$tab_title = __( 'Current text for voice-over', $this->plg_name );
						
						// * check edits
						$current_content_md5 = ( $post->post_content ) ? md5($post->post_content) : '';
						$saved_content_md5 = get_post_meta( $post->ID, '_' . $this->plg_name . '_content_md5', true );
						
						$post_excerpt_md5    = $this->excerpt_post( $post->ID );
						$current_excerpt_md5 = ( $post_excerpt_md5 ) ? md5($post_excerpt_md5) : '';
						$saved_excerpt_md5   = get_post_meta( $post->ID, '_' . $this->plg_name . '_excerpt_md5', true );
						
						$current_title_md5 = ( $post->post_title ) ? md5($post->post_title) : '';
						$saved_title_md5   = get_post_meta( $post->ID, '_' . $this->plg_name . '_title_md5', true );
						
						$new = '';
						$ch  = 0;
						if( 
							( $current_content_md5 && $saved_content_md5 && $current_content_md5 != $saved_content_md5 ) ||
							( $current_excerpt_md5 && $saved_excerpt_md5 && $current_excerpt_md5 != $saved_excerpt_md5 ) ||
							( $current_title_md5 && $saved_title_md5 && $current_title_md5 != $saved_title_md5 )
						)
						{
							$ch = 1;
						}
												
						$css = 'display:none;';

						// * if saved..
						$saved_text = get_post_meta( $post->ID, '_' . $this->plg_name . '_text', true );
						if( $saved_text ) 
						{
							$css = '';
							$tab_title .= ' (' . __( 'saved version', $this->plg_name ) . ')';
							$new = $content;
							$content = $saved_text;
						}
						
						// Add title
						if( $secions ) $secions .= "\n";
						
						// Only new
						if( ! $saved_text ) $content = $secions . $content;
						
						$html .= '
							<div class="zvk-block">
								<div class="zvk-title">'.$post->post_title.'</div>
								<div class="zvk-pid">post_id: <b>'.$post->ID.'</b></div>
								<div class="cls"></div>
								<div class="zvk-btn zvk-btn-active" onclick="code_tab(this, 0, '.$i.');" style="margin-top:20px;">'.$tab_title.'</div>
								<div class="zvk-btn" onclick="code_tab(this, 1, '.$i.');" style="margin-top:20px;">'.__( 'HTML (original)', $this->plg_name ).'</div>
								<div class="zvk-btn" onclick="code_tab(this, 2, '.$i.');" style="margin-top:20px;'.$css.'">'.__( 'Text (original)', $this->plg_name ).'</div>
								<textarea class="view_code_area edit_area view_content_area_'.$i.'" data-ids="'.$post->ID.'" name="edit_area_'.$i.'[]">' . $content . '</textarea>
								<textarea class="view_code_area view_html_area_'.$i.'" style="display:none;">' . $orig . '</textarea>
								<textarea class="view_code_area view_save_area_'.$i.'" style="display:none;">' . $new . '</textarea>
							</div>
						';		
						$i++;
					}
					else
					{
						$html .= '<div class="zvk-error">'.__( 'There is no content.', $this->plg_name ).'</div>';
					}
				}
				$html = $result . $html;
			}
			else
			{
				$html = '<div class="zvk-error">'.__( 'A document with such a post_id was not found in the system', $this->plg_name ).': <b>'.$posts_arr[0].'</b></div>';
			}
		}
		
		echo $html;
		exit;
	}
	
	// * Save audio config
	function save_audio_params_post( $post_id, $text )
	{
		$this->check_admin();
		
		$data_doc = [];
		
		if( $post_id > 0 ) 
		{
			// Replace &
			$text = str_replace( '@AMP@', '&', $text );
			
			$exceptions = [
				'_' . $this->plg_name . '_date',
				'_' . $this->plg_name . '_file',
				'_' . $this->plg_name . '_text_saved'
			];
			
			// * Meta renove
			$meta = get_post_meta( $post_id );
            foreach( $meta as $key => $item ) 
			{
				$m = explode('_' . $this->plg_name . '_', $key);
				if( ! empty( @$m[1] ) ) 
				{
					if( ! in_array($key, $exceptions) ) delete_post_meta( $post_id, $key );
				}
			}
			
			// * Update meta
			foreach( $this->audio_defailt_set as $k => $v )
			{
				$par = $this->data_cut_request( $k );
				if( empty($par) ) $par = $v;

				if( $k == 'audio_voice' ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					
					$data_doc['voice'] = $par;
				}
				elseif( $k == 'bitrate'   && $par != $this->audio_defailt_set['bitrate'] ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					$data_doc['bitrate'] = $par;
					
				}
				elseif( $k == 'format'    && $par != $this->audio_defailt_set['format'] ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					$data_doc['format'] = $par;
					
				}
				elseif( $k == 'speed'     && $par != $this->audio_defailt_set['speed'] ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					$data_doc['speed'] = $par;
				}
				elseif( $k == 'pitch'     && $par != $this->audio_defailt_set['pitch'] ) 
				{
					$data_doc['pitch'] = $par;
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
				}
				elseif( $k == 'emotion'   && $par != $this->audio_defailt_set['emotion'] ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					$data_doc['emotion'] = $par;
				}
				elseif( $k == 'sentence'  && $par != $this->audio_defailt_set['sentence'] ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					$data_doc['pause_sentence'] = $par;
					
				}
				elseif( $k == 'paragraph' && $par != $this->audio_defailt_set['paragraph'] ) 
				{
					update_post_meta( $post_id, '_' . $this->plg_name . '_' . $k, $par );
					$data_doc['pause_paragraph'] = $par;
				}
			}
						
			update_post_meta( $post_id, '_' . $this->plg_name . '_audio_lang',  $this->data_cut_request( 'audio_lang' ) );
			
			update_post_meta( $post_id, '_' . $this->plg_name . '_set_title',   (int) $this->data_cut_request( 'set_title' ) );
			update_post_meta( $post_id, '_' . $this->plg_name . '_set_excerpt', (int) $this->data_cut_request( 'set_excerpt' ) );
			update_post_meta( $post_id, '_' . $this->plg_name . '_set_text',    (int) $this->data_cut_request( 'set_text' ) );
			update_post_meta( $post_id, '_' . $this->plg_name . '_visible',     1 );
			
			$post_title_md5 = get_the_title( $post_id );
			$post_title_md5 = ( $post_title_md5 ) ? md5($post_title_md5) : '';
			
			$post_content_md5 = get_post($post_id);
			$post_content_md5 = ( $post_content_md5->post_content ) ? md5($post_content_md5->post_content) : '';
			
			$post_excerpt_md5 = $this->excerpt_post( $post_id );
			$post_excerpt_md5 = ( $post_excerpt_md5 ) ? md5($post_excerpt_md5) : '';

			update_post_meta( $post_id, '_' . $this->plg_name . '_text', $text );
			update_post_meta( $post_id, '_' . $this->plg_name . '_text_md5', md5( $text ) );
			
			update_post_meta( $post_id, '_' . $this->plg_name . '_title_md5',   $post_title_md5 );
			update_post_meta( $post_id, '_' . $this->plg_name . '_excerpt_md5', $post_excerpt_md5 );
			update_post_meta( $post_id, '_' . $this->plg_name . '_content_md5', $post_content_md5 );

			$data_doc['text'] = stripcslashes( htmlspecialchars_decode($text) );
		}
		
		return $data_doc;
	}
	
	// * First request
	function request_api( $data=[], $api_id, $post_id )
	{
		$this->check_admin();
		
		$url = $this->url_long_text;
		
		// check ID, stop process
		if( $api_id > 0 ) 
		{
			$url = $this->url_result;
			$data['id'] = $api_id;
		}
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_URL, $url );
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
		
		$response = curl_exec($ch);
		$response = json_decode($response, true);
		
		if( curl_errno($ch) ) 
		{
			$response['error'] = __( 'Connection error with the text recognition server.', $this->plg_name ) . curl_error($ch);
		}

		if( $this->request_log )
		{
			$path = $this->plg_dir . '/logs/' . $post_id . '.txt';
			
			if( ! $api_id  ) 
			{
				if( file_exists($path) ) unlink($path);
				
				$str  = "\r\n\r\n================================================\r\n\r\n";
				$str .= '1. '.__( 'LAUNCHING VOICEOVER', $this->plg_name ).', POST_ID ('.$post_id.') | ' . date('d.m.Y H:i:s', time() ) . "\r\n\r\n";
				$str .= __( 'SENT', $this->plg_name ) . ': ' . $url . "\r\n";
				foreach( $data as $k => $v )
				{
					if( $k != 'token' && $k != 'email' ) $str .= $k . ': '.$v . "\r\n";
				}
				
				$str .= "\r\n" . __( 'RECEIVED', $this->plg_name ) . ': ' . "\r\n";
				foreach( $response as $k => $v )
				{
					if( $k == 'cuts' ) $str .= $k . ': '.implode(',', $v) . "\r\n";
					else $str .= $k . ': '.$v . "\r\n";
				}
				file_put_contents( $path, $str . "\r\n", FILE_APPEND );
			}
			
			if( $api_id )
			{
				$str = '2. '.__( 'REQUEST TO END VOICEOVER', $this->plg_name ).', API_ID: '.$api_id.' | ' . date('d.m.Y H:i:s', time() ) . "\r\n";
				$str .= __( 'There are many identical queries, shown only with errors or results', $this->plg_name ) . "\r\n";
				if( @$response['error'] || @$response['file'] )
				{
					$str .= "\r\n" . __( 'SENT', $this->plg_name ) . ': ' . $url . "\r\n";
					foreach( $data as $k => $v )
					{
						if( $k != 'token' && $k != 'email' ) $str .= $k . ': '.$v . "\r\n";
					}
					
					$str .= "\r\n" . __( 'RECEIVED', $this->plg_name ) . ': ' . "\r\n";
					foreach( $response as $k => $v )
					{
						if( $k == 'cuts' ) $str .= $k . ': '.implode(', ', $v) . "\r\n";
						else $str .= $k . ': '.$v . "\r\n";
					}
					file_put_contents( $path, $str . "\r\n", FILE_APPEND );
				}
			}
		}
		
		if( $response['status'] == '-1' && @$data['id'] > 0 && empty( @$response['error'] ) )
		{
			$response['error'] = __( 'Voiceover error. Check if the language matches the text and the audio settings.', $this->plg_name ) 
			. '<br>'
			. $response['error'] = __( 'Some voices have limitations on audio settings, read more', $this->plg_name ) 
			. '<br> <a href="'.$this->url_dictors.'" target="_blank"> '.__( 'Online speakers and voice bots for text voiceover', $this->plg_name ).'</a>';
		}
		curl_close($ch);
		
		return $response;
	}
	
	function update_balans($balans)
	{
		$this->params['balans'] = $balans;
		update_option( $this->plg_name . '_params', $this->params );
	}
	
	
	function sound_process_func_ajax()
	{
		$this->check_admin();
		
		$result   = '';
		$error    = '';
		$file_src = '';
		$next     = 0;
	
		$post_id    = (int) $this->data_cut_request( 'next_id' );
		
		// Retry
		$rid_action = (int) $this->data_cut_request( 'rid_action' );
		if( $rid_action > 0 )
		{
			file_put_contents( $this->plg_dir . '/action_id', $post_id );
			file_put_contents( $this->plg_dir . '/action_request_id', $rid_action );
		}
		
		$text = htmlspecialchars( $_REQUEST['dts'] );

		if( $post_id > 0 )
		{
			$save_id = file_get_contents( $this->plg_dir . '/action_id' );
			
			$data_doc = [];
			$request  = [];
			
			// * save config for doc
			if( $save_id != $post_id ) 
			{
				$title  = get_the_title( $post_id );
				$result = '
					<div class="zvk-title">'.$title.'</div>
					<div class="zvk-pid">post_id: <b>'.$post_id.'</b></div>
					<div class="cls"></div>
				';
				
				$data_doc = $this->save_audio_params_post( $post_id, $text );
				$result .= '<div>'.__( 'Document configuration and text voiceover saved.', $this->plg_name ).'</div>';
				
				if( ! empty($text) )
				{
					file_put_contents( $this->plg_dir . '/action_id', $post_id );
					
					// * first request on id
					$data_doc['token'] = $this->key_array( $this->params, 'api_key');
					$data_doc['email'] = $this->key_array( $this->params, 'user_email');
					
					$request = $this->request_api( $data_doc, 0, $post_id );
					
					if( $request['status'] == 1 )
					{
						file_put_contents( $this->plg_dir . '/action_request_id', $request['id'] );
						
						$result .= '<div>'.__( 'Text processing on a remote server', $this->plg_name ).' (<b>api_id: '.$request['id'].'</b>)</div>';
					}
					else
					{
						$error = '<div><div class="zvk-error">'.$request['error'].'</div></div>';
						$result .= $error;
					}
				}
				else
				{
					$error = '<div><div class="zvk-error">'.__( 'There is no content for voiceover.', $this->plg_name ).'</div></div>';
					$result .= $error;
				}
			}
			
			$request_id = file_get_contents( $this->plg_dir . '/action_request_id' );
						
			// * Ожидаем ид очереди
			if( ! $error && $request_id > 0 )
			{
				$data_doc['token']  = $this->key_array( $this->params, 'api_key');
				$data_doc['email']  = $this->key_array( $this->params, 'user_email');
				
				// * запрос на файл
				$request = $this->request_api( $data_doc, $request_id, $post_id );
				

				if( $request['error'] )
				{
					$error = '<div><div class="zvk-error rid-'.$request_id.'">'.$request['error'].'</div></div>';
					$result .= $error;
				}
			}
			
			// * Ожидаем файл
			if( ! $error && @$request['file'] )
			{
				$file_audio = file_get_contents( $this->plg_dir . '/action_load' );
				
				if( ! $file_audio && @$request['file'] )
				{
					$file_audio = $request['file'];
					
					$this->update_balans( $request['balans'] );
					
					file_put_contents( $this->plg_dir . '/action_load', $file_audio );
					
					$file_src = 1;
				}
			}
		}
		else
		{
			$error =  '<div><div class="zvk-error">'.__( 'Post_id Error', $this->plg_name ).': <b>'.$post_id.'</b></div></div>';
			$result .= $error;
		}
		
		$repost = '';
		
		if( $error ) 
		{
			$next = 1;
			if( $request_id > 0 )
			{
				if( $rid_action > 0 )
				{
					$repost = '
						<div>
							<div class="repost_qw repost_qw-'.$request_id.'" data-request="'.$request_id.'" data-id="'.$post_id.'">'.__( 'Check the voiceover result again', $this->plg_name ).'</div>
						</div>
					';
				}
				else
				{
					$repost = '
						<div class="rid-target-'.$request_id.'">
							<div class="repost_qw repost_qw-'.$request_id.'" data-request="'.$request_id.'" data-id="'.$post_id.'">'.__( 'Check the voiceover result again', $this->plg_name ).'</div>
						</div>
					';
				}
			}
		}
		
		echo json_encode( 
			[
				'result'   => $result, 
				'error'    => $error,
				'next'     => $next, 
				'file_src' => $file_src,
				'repost'   => $repost
			]
		);
		exit;
	}
	
	
	function upload_audio_func_ajax()
	{
		$this->check_admin();
		
		$file_audio = file_get_contents( $this->plg_dir . '/action_load' );
		$post_id    = file_get_contents( $this->plg_dir . '/action_id' );
		
		$is_item    = (int) $this->data_cut_request( 'is_item' );
		
		$link = '<div style="margin-bottom:20px;"><a href="/wp-admin/admin.php?page='.$this->plg_name.'&tab=edit-'.$this->plg_name.'&id='.$post_id.'" target="href">Перейти к документу</a></div>';
		if( $is_item > 0 ) $link = '';

		$format = explode( '.', $file_audio );
		$extension = strtolower( end($format) );
		
		$result = '<div style="color:green;margin-bottom:5px;">'.__( 'The file is uploaded, the text is voiced.', $this->plg_name ).'</div>' . $link;
				
		if( $file_audio && in_array($extension, $this->audio_format_arr) )
		{
			$target_src = $this->plg_dir . 'uploads/' . $post_id . '.' . $extension;
			
			if( file_exists( $target_src ) ) unlink( $target_src );
			
			delete_post_meta( $post_id, '_' . $this->plg_name . '_date' );
			delete_post_meta( $post_id, '_' . $this->plg_name . '_file' );

			if( !copy( $file_audio, $target_src ) )
			{
				$result = '<div><div class="zvk-error">'.__( 'File copying error.', $this->plg_name ).'</div></div>';
			}
			else
			{
				update_post_meta( $post_id, '_' . $this->plg_name . '_date', date('d.m.Y H:i', time()) );
				update_post_meta( $post_id, '_' . $this->plg_name . '_file', $this->plg_url . 'uploads/' . $post_id . '.' . $extension );
				
				$old_text = stripcslashes( htmlspecialchars_decode(get_post_meta( $post_id, '_' . $this->plg_name . '_text', true )) );
				update_post_meta( $post_id, '_' . $this->plg_name . '_text_saved', $old_text );
			}
		}
		else
		{
			$result = '<div><div class="zvk-error">'.__( 'The file was not received or has an incorrect format.', $this->plg_name ).'</div></div>';
		}

		file_put_contents( $this->plg_dir . '/action_id', '0' );
		file_put_contents( $this->plg_dir . '/action_load', '0' );
		file_put_contents( $this->plg_dir . '/action_request_id', '0' );
		
		echo $result;

		exit;
	}
	
	function remove_audio_func_ajax()
	{
		$this->check_admin();

		$post_id = (int) $this->data_cut_request( 'pid' );
		if( $post_id > 0 )
		{
			$file = get_post_meta( $post_id, '_' . $this->plg_name . '_file', true );
			if( $file )
			{
				$f = explode('/', $file);
				if( end($f) && file_exists( $this->plg_dir . 'uploads/' . end($f) ) ) unlink( $this->plg_dir . 'uploads/' . end($f) );
			}
			
			$meta_remove = get_post_meta( $post_id );
			foreach( $meta_remove as $key => $item ) 
			{
				$m = explode('_' . $this->plg_name . '_', $key);
				if( ! empty( @$m[1] ) ) delete_post_meta( $post_id, $key );
			}
			
			if( file_exists( $this->plg_dir . 'logs/' . $post_id . '.txt' ) ) unlink( $this->plg_dir . 'logs/' . $post_id . '.txt' );
		}
		echo 'OK';
		exit;
	}
	
	
	function sound_start_func_ajax()
	{
		$this->check_admin();
		
		file_put_contents( $this->plg_dir . '/action_id', '0' );
		file_put_contents( $this->plg_dir . '/action_load', '0' );
		file_put_contents( $this->plg_dir . '/action_request_id', '0' );
		
		$result = '
			<h3>'.__( 'Voiceover started', $this->plg_name ).'</h3>
			<div>'
			.__( 'Do not close the browser - wait for the process to complete.', $this->plg_name )
			.'</div>
			<br>
			<div class="target_sound"></div>
		';
		
		echo $result;
		exit;
	}
	
	function get_data_section_doc($tab_values)
	{
		$this->check_admin();
		
		$set_title_selected   = $this->key_array( $this->params, 'set_title');
		$set_excerpt_selected = $this->key_array( $this->params, 'set_excerpt');
		$set_text_selected    = $this->key_array( $this->params, 'set_text');
		
		if( isset($tab_values[$this->plg_name . '_set_title']) ) $set_title_selected   = $tab_values[$this->plg_name . '_set_title'];
		if( isset($tab_values[$this->plg_name . '_set_excerpt']) ) $set_excerpt_selected = $tab_values[$this->plg_name . '_set_excerpt'];
		if( isset($tab_values[$this->plg_name . '_set_text']) ) $set_text_selected    = $tab_values[$this->plg_name . '_set_text'];
				
		$out = $this->html_tr(
			[
				'title'           => __( 'Voice the title', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => [
					[ '0', __( 'No', $this->plg_name ), '' ],
					[ '1', __( 'Yes', $this->plg_name ), '' ]
				],
				'selected'        => $set_title_selected,
				'name'            => 'set_title'
			]
		);
		
		$out .= $this->html_tr(
			[
				'title'           => __( 'Voice the passage (description)', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => [
					[ '0', __( 'No', $this->plg_name ), '' ],
					[ '1', __( 'Yes', $this->plg_name ), '' ]
				],
				'selected'        => $set_excerpt_selected,
				'name'            => 'set_excerpt'
				
			]
		);
		
		$out .= $this->html_tr(
			[
				'title'           => __( 'Voice the text', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => [
					[ '0', __( 'No', $this->plg_name ), '' ],
					[ '1', __( 'Yes', $this->plg_name ), '' ]
				],
				'selected'        => $set_text_selected ,
				'name'            => 'set_text',
			]
		);
		
		return $out;
	}
	
	
	function get_data_audio($tab_values)
	{
		$this->check_admin();
		
		$out = '';
		
		$audio_lang_selected  = $this->key_array( $this->params, 'audio_lang');
		$audio_voice_selected = $this->key_array( $this->params, 'audio_voice');
		$bitrate_selected     = $this->key_array( $this->params, 'bitrate');
		$sentence_selected    = $this->key_array( $this->params, 'sentence');
		$paragraph_selected   = $this->key_array( $this->params, 'paragraph');
		$speed_selected       = $this->key_array( $this->params, 'speed');
		$pitch_selected       = $this->key_array( $this->params, 'pitch');
		
		if( isset($tab_values[$this->plg_name . '_audio_lang']) )  $audio_lang_selected  = @$tab_values[$this->plg_name . '_audio_lang'];
		if( isset($tab_values[$this->plg_name . '_audio_voice']) ) $audio_voice_selected = @$tab_values[$this->plg_name . '_audio_voice'];
		if( isset($tab_values[$this->plg_name . '_bitrate']) )     $bitrate_selected     = @$tab_values[$this->plg_name . '_bitrate'];
		if( isset($tab_values[$this->plg_name . '_sentence']) )    $sentence_selected    = @$tab_values[$this->plg_name . '_sentence'];
		if( isset($tab_values[$this->plg_name . '_paragraph']) )   $paragraph_selected   = @$tab_values[$this->plg_name . '_paragraph'];
		if( isset($tab_values[$this->plg_name . '_speed']) )       $speed_selected       = @$tab_values[$this->plg_name . '_speed'];
		if( isset($tab_values[$this->plg_name . '_pitch']) )       $pitch_selected       = @$tab_values[$this->plg_name . '_pitch'];
				
		$audio_lang      = [];
		$audio_voice_arr = [];
		$i = 0;
		foreach( @$this->params['voices_arr'] as $lang => $arr )
		{
			$audio_lang[] = [ $lang, $lang, '' ];
			if( !$i ) $audio_voice_arr = $arr;
			
			if( $lang == $this->key_array( $this->params, 'audio_lang') ) $audio_voice_arr = $arr;
			
			if( @$tab_values[$this->plg_name . '_audio_lang'] && $lang == $tab_values[$this->plg_name . '_audio_lang'] ) $audio_voice_arr = $arr;
			
			$i++;
		}

		$out .= $this->html_tr(
			[
				'title'           => __( 'Voice language', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $audio_lang,
				'selected'        => $audio_lang_selected,
				'name'            => 'audio_lang'
			]
		);
		
		$audio_voice = [];
		foreach( $audio_voice_arr as $k => $arr )
		{
			$sex = __( '(m)', $this->plg_name );
			if( $arr->sex == 'female' ) $sex = __( '(w)', $this->plg_name );
			
			$audio_voice[] = [ $arr->voice, $arr->voice . ' '.$sex, '' ];
		}
		
		$out .= $this->html_tr(
			[
				'title'           => __( 'Default voice', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $audio_voice,
				'selected'        => $audio_voice_selected,
				'name'            => 'audio_voice'
			]
		);
		
		//
		/*
		$emotion_arr = [];
		foreach( $this->audio_emotion_arr as $v )
		{
			$emotion_arr[] = [ $v, $v, '' ];
		}
		$out .= $this->html_tr(
			[
				'title'           => __( 'Emotions', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $emotion_arr,
				'selected'        => $this->key_array( $this->params, 'emotion'),
				'name'            => 'emotion'
			]
		);
		*/
				
		$bitrate_arr = [];
		foreach( $this->audio_bitrate_arr as $v )
		{
			$bitrate_arr[] = [ $v, $v, '' ];
		}
		$out .= $this->html_tr(
			[
				'title'           => __( 'Bitrate', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $bitrate_arr,
				'selected'        => $bitrate_selected,
				'name'            => 'bitrate'
			]
		);
						
		$sentence_arr = [];
		foreach( $this->audio_sentence_arr as $v )
		{
			$sentence_arr[] = [ $v, $v, '' ];
		}
		$out .= $this->html_tr(
			[
				'title'           => __( 'Pauses between sentences', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $sentence_arr,
				'selected'        => $sentence_selected,
				'name'            => 'sentence'
			]
		);
				
		$paragraph_arr = [];
		foreach( $this->audio_paragraph_arr as $v )
		{
			$paragraph_arr[] = [ $v, $v, '' ];
		}
		$out .= $this->html_tr(
			[
				'title'           => __( 'Pauses between paragraphs', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $paragraph_arr,
				'selected'        => $paragraph_selected,
				'name'            => 'paragraph'
			]
		);
				
		$speed_arr = [];
		foreach( $this->audio_speed_arr as $v )
		{
			$speed_arr[] = [ $v, $v, '' ];
		}
		$out .= $this->html_tr(
			[
				'title'           => __( 'Playback speed', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $speed_arr,
				'selected'        => $speed_selected,
				'name'            => 'speed'
			]
		);
				
		$pitch_arr = [];
		foreach( $this->audio_pitch_arr as $v )
		{
			$pitch_arr[] = [ $v, $v, '' ];
		}
		$out .= $this->html_tr(
			[
				'title'           => __( 'Voice pitch', $this->plg_name ),
				'field'           => 'select',
				'select_arr'      => $pitch_arr,
				'selected'        => $pitch_selected,
				'name'            => 'pitch'
			]
		);
		
		$out .= $this->html_tr_wrapper_empty( 
			'<div class="default_code_div" ><a href="javascript:void(0);" class="default_audio" style="text-decoration:none;display:inline-block;border-bottom:1px dashed #999;" >'.__( 'Restore default audio values', $this->plg_name ).'</a></div>' 
		);

		return $out;
	}
	
	function select_voice_func_ajax()
	{
		$this->check_admin();
		
		$country = $this->data_cut_request( 'country' );
		
		$out = '';
		if( $country )
		{
			foreach( @$this->params['voices_arr'] as $lang => $arr )
			{
				if( $country == $lang )
				{
					foreach( $arr as $k => $v )
					{
						$sex = __( '(m)', $this->plg_name );
						if( $v->sex == 'female' ) $sex = __( '(w)', $this->plg_name );
						
						$out .= '<option value="'.$v->voice.'">'.$v->voice . ' '.$sex.'</option>';
					}
					break;
				}
			}
		}
		
		echo $out;
		exit;
	}
	
	function cur_plg_short_func( $attr )
	{
		if( is_admin() ) return;

		$html = '';

		$post_id = get_the_ID();
		$post_meta = get_post_meta( $post_id );
		
		if( @$post_meta['_' . $this->plg_name . '_visible'][0] && @$post_meta['_' . $this->plg_name . '_file'][0] && $this->params['code_player'] )
		{
			$style = '';
			if( @$this->params['code_css'] )
			{
				$style =  '<style>'.stripcslashes( htmlspecialchars_decode( $this->params['code_css'] ) ).'</style>';
			}
			
			if( strpos( $this->params['code_player'], '[AUDIO_FILE_PATH]' ) !== false )
			{
				$html = stripcslashes(
					htmlspecialchars_decode(
						str_replace( 
							'[AUDIO_FILE_PATH]', 
							$post_meta['_' . $this->plg_name . '_file'][0], 
							$this->params['code_player'] 
						)
					)
				);
				
			}
			else
			{
				// If [AUDIO_FILE_PATH] string not found
				$html = '
					<div class="zg-player">
						<div style="color:red;font-size:12px; line-height:1.3em;">'.__( 'The default player is shown.', $this->plg_name ).' <br>'.__( 'Check the player code settings (autocorrect).', $this->plg_name ).'<br><br></div>
						<audio preload="auto" controls controlsList="nodownload">
							<source src="'.$post_meta['_' . $this->plg_name . '_file'][0].'">
						</audio>
					</div>
				';
			}
			
			// WP AUDIO
			if( @$this->params['player'] > 0 )
			{
				ob_start();
					echo  do_shortcode( $html );
					$html = ob_get_contents();
				ob_end_clean();
			}
		}
		return $style . $html;
	}
	
	// ajax player preview
	function preview_player_func_ajax()
	{
		$html   = stripcslashes( htmlspecialchars_decode( $_REQUEST['c'] ) );
		$style  = '<style>' . stripcslashes( htmlspecialchars_decode( $_REQUEST['s'] ) ) . '</style>';
		$player = (int) $this->data_cut_request( 't' );
		
		if( $player )
		{
			ob_start();
				echo  do_shortcode( $html );
				$html = ob_get_contents();
			ob_end_clean();
		}
		
		echo $style . $html;
		
		exit;
	}
}
